@@ -15,7 +15,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
 #endif
 
 #include <php.h>
@@ -24,18 +24,19 @@
 #include "phongo_compat.h"
 #include "php_phongo.h"
 
-zend_class_entry *php_phongo_writeconcern_ce;
+zend_class_entry* php_phongo_writeconcern_ce;
 
 /* {{{ proto void MongoDB\Driver\WriteConcern::__construct(integer|string $w[, integer $wtimeout[, boolean $journal]])
    Constructs a new WriteConcern */
 static PHP_METHOD(WriteConcern, __construct)
 {
-	php_phongo_writeconcern_t *intern;
-	zend_error_handling       error_handling;
-	zval                     *w, *journal;
-	phongo_long               wtimeout = 0;
-	SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
-
+	php_phongo_writeconcern_t* intern;
+	zend_error_handling        error_handling;
+	zval *                     w, *journal;
+	phongo_long                wtimeout = 0;
+	SUPPRESS_UNUSED_WARNING(return_value)
+	SUPPRESS_UNUSED_WARNING(return_value_ptr)
+	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
@@ -46,7 +47,6 @@ static PHP_METHOD(WriteConcern, __construct)
 	}
 	zend_restore_error_handling(&error_handling TSRMLS_CC);
 
-
 	intern->write_concern = mongoc_write_concern_new();
 
 	if (Z_TYPE_P(w) == IS_LONG) {
@@ -62,11 +62,11 @@ static PHP_METHOD(WriteConcern, __construct)
 			mongoc_write_concern_set_wtag(intern->write_concern, Z_STRVAL_P(w));
 		}
 	} else {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected w to be integer or string, %s given", zend_get_type_by_const(Z_TYPE_P(w)));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected w to be integer or string, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(w));
 		return;
 	}
 
-	switch(ZEND_NUM_ARGS()) {
+	switch (ZEND_NUM_ARGS()) {
 		case 3:
 			if (Z_TYPE_P(journal) != IS_NULL) {
 #ifdef ZEND_ENGINE_3
@@ -95,9 +95,10 @@ static PHP_METHOD(WriteConcern, __construct)
    Returns the WriteConcern "w" option */
 static PHP_METHOD(WriteConcern, getW)
 {
-	php_phongo_writeconcern_t *intern;
-	const char *wtag;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_writeconcern_t* intern;
+	const char*                wtag;
+	SUPPRESS_UNUSED_WARNING(return_value_ptr)
+	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
@@ -126,8 +127,9 @@ static PHP_METHOD(WriteConcern, getW)
    Returns the WriteConcern "wtimeout" option */
 static PHP_METHOD(WriteConcern, getWtimeout)
 {
-	php_phongo_writeconcern_t *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_writeconcern_t* intern;
+	SUPPRESS_UNUSED_WARNING(return_value_ptr)
+	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
@@ -142,8 +144,9 @@ static PHP_METHOD(WriteConcern, getWtimeout)
    Returns the WriteConcern "journal" option */
 static PHP_METHOD(WriteConcern, getJournal)
 {
-	php_phongo_writeconcern_t *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_writeconcern_t* intern;
+	SUPPRESS_UNUSED_WARNING(return_value_ptr)
+	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
@@ -163,8 +166,9 @@ static PHP_METHOD(WriteConcern, getJournal)
    with no write concern URI options). */
 static PHP_METHOD(WriteConcern, isDefault)
 {
-	php_phongo_writeconcern_t *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_writeconcern_t* intern;
+	SUPPRESS_UNUSED_WARNING(return_value_ptr)
+	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_WRITECONCERN_OBJ_P(getThis());
 
@@ -175,17 +179,102 @@ static PHP_METHOD(WriteConcern, isDefault)
 	RETURN_BOOL(mongoc_write_concern_is_default(intern->write_concern));
 } /* }}} */
 
+static HashTable* php_phongo_write_concern_get_properties_hash(zval* object, bool is_debug TSRMLS_DC) /* {{{ */
+{
+	php_phongo_writeconcern_t* intern;
+	HashTable*                 props;
+	const char*                wtag;
+	int32_t                    w;
+	int32_t                    wtimeout;
+
+	intern = Z_WRITECONCERN_OBJ_P(object);
+
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 4);
+
+	if (!intern->write_concern) {
+		return props;
+	}
+
+	wtag     = mongoc_write_concern_get_wtag(intern->write_concern);
+	w        = mongoc_write_concern_get_w(intern->write_concern);
+	wtimeout = mongoc_write_concern_get_wtimeout(intern->write_concern);
+
+#if PHP_VERSION_ID >= 70000
+	{
+		zval z_w;
+
+		if (wtag) {
+			ZVAL_STRING(&z_w, wtag);
+			zend_hash_str_update(props, "w", sizeof("w") - 1, &z_w);
+		} else if (mongoc_write_concern_get_wmajority(intern->write_concern)) {
+			ZVAL_STRING(&z_w, PHONGO_WRITE_CONCERN_W_MAJORITY);
+			zend_hash_str_update(props, "w", sizeof("w") - 1, &z_w);
+		} else if (w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
+			ZVAL_LONG(&z_w, w);
+			zend_hash_str_update(props, "w", sizeof("w") - 1, &z_w);
+		}
+
+		if (mongoc_write_concern_journal_is_set(intern->write_concern)) {
+			zval z_j;
+
+			ZVAL_BOOL(&z_j, mongoc_write_concern_get_journal(intern->write_concern));
+			zend_hash_str_update(props, "j", sizeof("j") - 1, &z_j);
+		}
+
+		if (wtimeout != 0) {
+			zval z_wtimeout;
+
+			ZVAL_LONG(&z_wtimeout, wtimeout);
+			zend_hash_str_update(props, "wtimeout", sizeof("wtimeout") - 1, &z_wtimeout);
+		}
+#else
+	{
+		zval* z_w;
+
+		if (wtag) {
+			MAKE_STD_ZVAL(z_w);
+			ZVAL_STRING(z_w, wtag, 1);
+			zend_hash_update(props, "w", sizeof("w"), &z_w, sizeof(z_w), NULL);
+		} else if (mongoc_write_concern_get_wmajority(intern->write_concern)) {
+			MAKE_STD_ZVAL(z_w);
+			ZVAL_STRING(z_w, PHONGO_WRITE_CONCERN_W_MAJORITY, 1);
+			zend_hash_update(props, "w", sizeof("w"), &z_w, sizeof(z_w), NULL);
+		} else if (w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
+			MAKE_STD_ZVAL(z_w);
+			ZVAL_LONG(z_w, w);
+			zend_hash_update(props, "w", sizeof("w"), &z_w, sizeof(z_w), NULL);
+		}
+
+		if (mongoc_write_concern_journal_is_set(intern->write_concern)) {
+			zval* z_j;
+
+			MAKE_STD_ZVAL(z_j);
+			ZVAL_BOOL(z_j, mongoc_write_concern_get_journal(intern->write_concern));
+			zend_hash_update(props, "j", sizeof("j"), &z_j, sizeof(z_j), NULL);
+		}
+
+		if (wtimeout != 0) {
+			zval* z_wtimeout;
+
+			MAKE_STD_ZVAL(z_wtimeout);
+			ZVAL_LONG(z_wtimeout, wtimeout);
+			zend_hash_update(props, "wtimeout", sizeof("wtimeout"), &z_wtimeout, sizeof(z_wtimeout), NULL);
+		}
+#endif
+	}
+
+	return props;
+} /* }}} */
+
 /* {{{ proto array MongoDB\Driver\WriteConcern::bsonSerialize()
 */
 static PHP_METHOD(WriteConcern, bsonSerialize)
 {
-	const mongoc_write_concern_t *write_concern = phongo_write_concern_from_zval(getThis() TSRMLS_CC);
-
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
-	php_phongo_write_concern_to_zval(return_value, write_concern);
+	ZVAL_ARR(return_value, php_phongo_write_concern_get_properties_hash(getThis(), true TSRMLS_CC));
 	convert_to_object(return_value);
 } /* }}} */
 
@@ -200,25 +289,32 @@ ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_writeconcern_me[] = {
-	PHP_ME(WriteConcern, __construct, ai_WriteConcern___construct, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getW, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, isDefault, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, bsonSerialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	/* clang-format off */
+	PHP_ME(WriteConcern, __construct, ai_WriteConcern___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getW, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, isDefault, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, bsonSerialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
+	/* clang-format on */
 };
 /* }}} */
 
 /* {{{ MongoDB\Driver\WriteConcern object handlers */
 static zend_object_handlers php_phongo_handler_writeconcern;
 
-static void php_phongo_writeconcern_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
+static void php_phongo_writeconcern_free_object(phongo_free_object_arg* object TSRMLS_DC) /* {{{ */
 {
-	php_phongo_writeconcern_t *intern = Z_OBJ_WRITECONCERN(object);
+	php_phongo_writeconcern_t* intern = Z_OBJ_WRITECONCERN(object);
 
 	zend_object_std_dtor(&intern->std TSRMLS_CC);
 
+	if (intern->properties) {
+		zend_hash_destroy(intern->properties);
+		FREE_HASHTABLE(intern->properties);
+	}
+
 	if (intern->write_concern) {
 		mongoc_write_concern_destroy(intern->write_concern);
 	}
@@ -228,9 +324,9 @@ static void php_phongo_writeconcern_free_object(phongo_free_object_arg *object T
 #endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_writeconcern_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_writeconcern_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
 {
-	php_phongo_writeconcern_t *intern = NULL;
+	php_phongo_writeconcern_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_writeconcern_t, class_type);
 
@@ -252,29 +348,23 @@ static phongo_create_object_retval php_phongo_writeconcern_create_object(zend_cl
 #endif
 } /* }}} */
 
-static HashTable *php_phongo_writeconcern_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_debug_info(zval* object, int* is_temp TSRMLS_DC) /* {{{ */
 {
-#if PHP_VERSION_ID >= 70000
-	zval                          retval;
-#else
-	zval                          retval = zval_used_for_init;
-#endif
-	const mongoc_write_concern_t *write_concern = phongo_write_concern_from_zval(object TSRMLS_CC);
-
-
 	*is_temp = 1;
-	php_phongo_write_concern_to_zval(&retval, write_concern);
+	return php_phongo_write_concern_get_properties_hash(object, true TSRMLS_CC);
+} /* }}} */
 
-	return Z_ARRVAL(retval);
+static HashTable* php_phongo_writeconcern_get_properties(zval* object TSRMLS_DC) /* {{{ */
+{
+	return php_phongo_write_concern_get_properties_hash(object, false TSRMLS_CC);
 } /* }}} */
-/* }}} */
 
 void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteConcern", php_phongo_writeconcern_me);
-	php_phongo_writeconcern_ce = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_writeconcern_ce                = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_writeconcern_ce->create_object = php_phongo_writeconcern_create_object;
 	PHONGO_CE_FINAL(php_phongo_writeconcern_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_writeconcern_ce);
@@ -283,9 +373,10 @@ void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS) /* {{{ */
 
 	memcpy(&php_phongo_handler_writeconcern, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_writeconcern.get_debug_info = php_phongo_writeconcern_get_debug_info;
+	php_phongo_handler_writeconcern.get_properties = php_phongo_writeconcern_get_properties;
 #if PHP_VERSION_ID >= 70000
 	php_phongo_handler_writeconcern.free_obj = php_phongo_writeconcern_free_object;
-	php_phongo_handler_writeconcern.offset = XtOffsetOf(php_phongo_writeconcern_t, std);
+	php_phongo_handler_writeconcern.offset   = XtOffsetOf(php_phongo_writeconcern_t, std);
 #endif
 
 	zend_declare_class_constant_stringl(php_phongo_writeconcern_ce, ZEND_STRL("MAJORITY"), ZEND_STRL(PHONGO_WRITE_CONCERN_W_MAJORITY) TSRMLS_CC);
